@@ -3,12 +3,13 @@
 """Directory entry operations with PyFAT."""
 import posixpath
 import struct
+import warnings
 
 from time import timezone
 
 from pyfatfs.DosDateTime import DosDateTime
 from pyfatfs.EightDotThree import EightDotThree
-from pyfatfs._exceptions import PyFATException, NotAnLFNEntryException,\
+from pyfatfs._exceptions import PyFATException, NotAnLFNEntryException, \
     BrokenLFNEntryException
 from pyfatfs import FAT_OEM_ENCODING, FAT_LFN_ENCODING
 
@@ -46,6 +47,8 @@ class FATDirectoryEntry:
     FAT_DIRECTORY_LAYOUT = "<11sBBBHHHHHHHL"
     #: Size of a directory entry header in bytes
     FAT_DIRECTORY_HEADER_SIZE = struct.calcsize(FAT_DIRECTORY_LAYOUT)
+    #: Maximum allowed file size, dictated by size of DIR_FileSize
+    MAX_FILE_SIZE = 0xFFFFFFFF
     #: Directory entry headers
     FAT_DIRECTORY_VARS = ["DIR_Name", "DIR_Attr", "DIR_NTRes",
                           "DIR_CrtTimeTenth", "DIR_CrtTime",
@@ -54,12 +57,15 @@ class FATDirectoryEntry:
                           "DIR_WrtDate", "DIR_FstClusLO",
                           "DIR_FileSize"]
 
-    def __init__(self, DIR_Name: EightDotThree, DIR_Attr: int,
+    def __init__(self,
+                 DIR_Name: EightDotThree, DIR_Attr: int,
                  DIR_NTRes: int, DIR_CrtTimeTenth: int,
                  DIR_CrtTime: int, DIR_CrtDate: int, DIR_LstAccessDate: int,
                  DIR_FstClusHI: int, DIR_WrtTime: int, DIR_WrtDate: int,
                  DIR_FstClusLO: int, DIR_FileSize: int,
-                 encoding: str = FAT_OEM_ENCODING, lfn_entry=None):
+                 encoding: str = FAT_OEM_ENCODING,
+                 fs: "pyfatfs.PyFat.PyFat" = None,  # noqa: F821
+                 lazy_load: bool = False, lfn_entry=None):
         """FAT directory entry constructor.
 
         :param DIR_Name: `EightDotThree` class instance
@@ -77,6 +83,8 @@ class FATDirectoryEntry:
         :param encoding: Encoding of filename
         :param lfn_entry: FATLongDirectoryEntry instance or None
         """
+        self.__filesize = 0
+
         self.name: EightDotThree = DIR_Name
         self.attr = int(DIR_Attr)
         self.ntres = int(DIR_NTRes)
@@ -90,15 +98,48 @@ class FATDirectoryEntry:
         self.fstcluslo = int(DIR_FstClusLO)
         self.filesize = int(DIR_FileSize)
 
+        self.__lazy_load = lazy_load
+        self.__fs = fs
+
         self._parent = None
 
         # Handle LFN entries
         self.lfn_entry = None
-        self.set_lfn_entry(lfn_entry)
+        try:
+            self.set_lfn_entry(lfn_entry)
+        except BrokenLFNEntryException:
+            warnings.warn("Broken LFN entry detected, omitting "
+                          "long file name.")
 
         self.__dirs = []
         self.__encoding = encoding
 
+    @property
+    def _encoding(self):
+        return self.__encoding
+
+    @property
+    def filesize(self):
+        """Size of the file in bytes.
+
+        :getter: Get the currently set filesize in bytes
+        :setter: Set new filesize. FAT chain must be extended
+                 separately. Raises `PyFATException` with
+                 `errno=E2BIG` if filesize is larger than
+                 `FATDirectoryEntry.MAX_FILE_SIZE`.
+        :type: int
+        """
+        return self.__filesize
+
+    @filesize.setter
+    def filesize(self, size: int):
+        if size > self.MAX_FILE_SIZE:
+            raise PyFATException(f"Specified file size {size} too large "
+                                 f"for FAT-based filesystems.",
+                                 errno=errno.E2BIG)
+
+        self.__filesize = size
+
     @staticmethod
     def new(name: EightDotThree, tz: timezone, encoding: str,
             attr: int = 0, ntres: int = 0, cluster: int = 0,
@@ -182,6 +223,9 @@ class FATDirectoryEntry:
 
         :returns: Entry size in bytes as int
         """
+        if self.is_directory():
+            self.__populate_dirs()
+
         sz = self.FAT_DIRECTORY_HEADER_SIZE
         if isinstance(self.lfn_entry, FATLongDirectoryEntry):
             sz *= len(self.lfn_entry.lfn_entries)
@@ -194,6 +238,11 @@ class FATDirectoryEntry:
 
         :returns: Filesize or directory entry size in bytes as int
         """
+        import warnings
+        warnings.warn(f"{self.__class__}.get_size is deprecated, this "
+                      f"method will be removed in PyFatFS 2.0; please "
+                      f"use the filesize property instead!",
+                      DeprecationWarning)
         return self.filesize
 
     def set_size(self, size: int):
@@ -201,6 +250,11 @@ class FATDirectoryEntry:
 
         :param size: `int`: File size in bytes
         """
+        import warnings
+        warnings.warn(f"{self.__class__}.set_size is deprecated, this "
+                      f"method will be removed in PyFatFS 2.0; please "
+                      f"use the filesize property instead!",
+                      DeprecationWarning)
         self.filesize = size
 
     def get_cluster(self):
@@ -343,6 +397,7 @@ class FATDirectoryEntry:
     def is_empty(self):
         """Determine if directory does not contain any directories."""
         self._verify_is_directory()
+        self.__populate_dirs()
 
         for d in self.__dirs:
             if d.is_special():
@@ -351,9 +406,20 @@ class FATDirectoryEntry:
 
         return True
 
+    def __populate_dirs(self):
+        if self.__lazy_load is False:
+            return
+
+        clus = self.get_cluster()
+        self.__dirs = self.__fs.parse_dir_entries_in_cluster_chain(clus)
+        for dir_entry in self.__dirs:
+            dir_entry._add_parent(self)
+        self.__lazy_load = False
+
     def _get_entries_raw(self):
         """Get a full list of entries in current directory."""
         self._verify_is_directory()
+        self.__populate_dirs()
 
         return self.__dirs
 
@@ -420,6 +486,9 @@ class FATDirectoryEntry:
         :returns: tuple: root (current path, full),
                          dirs (all dirs), files (all files)
         """
+        self._verify_is_directory()
+        self.__populate_dirs()
+
         root = self.get_full_path()
         dirs, files, _ = self.get_entries()
 
@@ -434,7 +503,7 @@ class FATDirectoryEntry:
 
             yield from d.walk()
 
-    def add_subdirectory(self, dir_entry):
+    def add_subdirectory(self, dir_entry, recursive: bool = True):
         """Register a subdirectory in current directory entry.
 
         :param dir_entry: FATDirectoryEntry
@@ -444,6 +513,7 @@ class FATDirectoryEntry:
         """
         # Check if current dir entry is even a directory!
         self._verify_is_directory()
+        self.__populate_dirs()
 
         dir_entry._add_parent(self)
         self.__dirs += [dir_entry]
