@@ -13,7 +13,7 @@ import time
 import warnings
 
 from contextlib import contextmanager
-from io import BufferedReader, FileIO, open, BytesIO, IOBase, SEEK_END
+from io import FileIO, open, BytesIO, IOBase, SEEK_END
 from os import PathLike
 from typing import Union
 
@@ -34,7 +34,8 @@ def _readonly_check(func):
             return func(*args, **kwargs)
         else:
             raise PyFATException("Filesystem has been opened read-only, not "
-                                 "able to perform a write operation!")
+                                 "able to perform a write operation!",
+                                 errno=errno.EROFS)
 
     return _wrapper
 
@@ -98,7 +99,8 @@ class PyFat(object):
 
     def __init__(self,
                  encoding: str = 'ibm437',
-                 offset: int = 0):
+                 offset: int = 0,
+                 lazy_load: bool = True):
         """Set up PyFat class instance.
 
         :param encoding: Define encoding to use for filenames
@@ -121,12 +123,14 @@ class PyFat(object):
         self.initialized = False
         self.encoding = encoding
         self.is_read_only = True
+        self.lazy_load = lazy_load
         self.__lock = threading.Lock()
 
     def __set_fp(self, fp: Union[IOBase, BytesIO]):
-        if isinstance(self.__fp, BufferedReader):
+        if self.__fp is not None:
             raise PyFATException("Cannot overwrite existing file handle, "
-                                 "create new class instance of PyFAT.")
+                                 "create new class instance of PyFAT.",
+                                 errno=errno.EMFILE)
         self.__fp = fp
 
     def __seek(self, address: int):
@@ -239,8 +243,6 @@ class PyFat(object):
             self._mark_dirty()
 
         # Parse root directory
-        # TODO: Inefficient to always recursively parse the root dir.
-        #       It would make sense to parse it on demand instead.
         self.parse_root_dir()
 
     def open(self, filename: Union[str, PathLike], read_only: bool = False):
@@ -266,7 +268,6 @@ class PyFat(object):
         """Retrieve path of opened filesystem."""
         return self.__fp.name
 
-    @_init_check
     def _get_total_sectors(self):
         """Get total number of sectors for all FAT sizes."""
         if self.bpb_header["BPB_TotSec16"] != 0:
@@ -647,14 +648,16 @@ class PyFat(object):
 
         # Follow root directory cluster chain
         for dir_entry in self.parse_dir_entries_in_cluster_chain(root_cluster):
-            self.root_dir.add_subdirectory(dir_entry)
+            self.root_dir.add_subdirectory(dir_entry,
+                                           recursive=not self.lazy_load)
 
     def parse_root_dir(self):
         """Parse root directory entry."""
         root_dir_sfn = EightDotThree()
         root_dir_sfn.set_str_name("")
         dir_attr = FATDirectoryEntry.ATTR_DIRECTORY
-        self.root_dir = FATDirectoryEntry(DIR_Name=root_dir_sfn,
+        self.root_dir = FATDirectoryEntry(fs=self,
+                                          DIR_Name=root_dir_sfn,
                                           DIR_Attr=dir_attr,
                                           DIR_NTRes=0,
                                           DIR_CrtTimeTenth=0,
@@ -748,24 +751,27 @@ class PyFat(object):
                 # Ignore incomplete LFN entries altogether
                 tmp_lfn_entry = None
 
-            dir_entry = FATDirectoryEntry(encoding=self.encoding,
+            dir_entry = FATDirectoryEntry(fs=self,
+                                          encoding=self.encoding,
                                           lfn_entry=tmp_lfn_entry,
+                                          lazy_load=self.lazy_load,
                                           **dir_hdr)
             dir_entries += [dir_entry]
 
-            if dir_entry.is_directory() and not dir_entry.is_special():
-                # Iterate all subdirectories except for dot and dotdot
-                cluster = dir_entry.get_cluster()
-                subdirs = self.parse_dir_entries_in_cluster_chain(cluster)
-                for d in subdirs:
-                    dir_entry.add_subdirectory(d)
+            if not self.lazy_load:
+                if dir_entry.is_directory() and not dir_entry.is_special():
+                    # Iterate all subdirectories except for dot and dotdot
+                    cluster = dir_entry.get_cluster()
+                    subdirs = self.parse_dir_entries_in_cluster_chain(cluster)
+                    for d in subdirs:
+                        dir_entry.add_subdirectory(d)
 
             # Reset temporary LFN entry
             tmp_lfn_entry = FATLongDirectoryEntry()
 
         return dir_entries, tmp_lfn_entry
 
-    def parse_dir_entries_in_cluster_chain(self, cluster):
+    def parse_dir_entries_in_cluster_chain(self, cluster) -> list:
         """Parse directory entries while following given cluster chain."""
         dir_entries = []
         tmp_lfn_entry = FATLongDirectoryEntry()
@@ -857,11 +863,7 @@ class PyFat(object):
         returns: `str`: Any of PyFat.FAT_TYPE_FAT12, PyFat.FAT_TYPE_FAT16
                  or PyFat.FAT_TYPE_FAT32
         """
-        if self.bpb_header["BPB_TotSec16"] != 0:
-            total_sectors = self.bpb_header["BPB_TotSec16"]
-        else:
-            total_sectors = self.bpb_header["BPB_TotSec32"]
-
+        total_sectors = self._get_total_sectors()
         rsvd_sectors = self.bpb_header["BPB_RsvdSecCnt"]
         fat_sz = self.bpb_header["BPB_NumFATs"] * self._fat_size
         root_dir_sectors = self.root_dir_sectors
@@ -1096,7 +1098,14 @@ class PyFat(object):
                 (4194304, 64)   # disks up to   2   GB, 32k cluster
             ],
             PyFat.FAT_TYPE_FAT12: [
-                (32768, 64)
+                (4084, 1),      # disks up to   2   MB, .5k cluster
+                (8168, 2),      # disks up to   4   MB,  1k cluster
+                (16336, 4),     # disks up to   8   MB,  2k cluster
+                (32672, 8),     # disks up to  16   MB,  4k cluster
+                (65344, 16),    # disks up to  32   MB,  8k cluster
+                (130688, 32),   # disks up to  64   MB, 16k cluster
+                (261376, 64),   # disks up to 128   MB, 32k cluster
+                (522752, 128)   # disks up to 256   MB, 64k cluster
             ]
         }
         sec_per_clus = 0
@@ -1134,7 +1143,10 @@ class PyFat(object):
         elif fat_type == PyFat.FAT_TYPE_FAT16:
             root_ent_cnt = 512
         else:
-            root_ent_cnt = 224  # randomly picked, fine if sector_size is 512
+            if sector_size == 512:
+                root_ent_cnt = 224  # Floppy typical
+            else:
+                root_ent_cnt = 512
 
         rsvd_sec_cnt = 32 if fat_type == PyFat.FAT_TYPE_FAT32 else 1
 
